/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.credential;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link AcquisitionFlow}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new AcquisitionFlow.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableAcquisitionFlow.of()}.
 */
@Generated(from = "AcquisitionFlow", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableAcquisitionFlow implements AcquisitionFlow {
  private final String redirectUrl;
  private final Type type;
  private final @Nullable CredentialFlowState state;

  @SuppressWarnings("unchecked") // safe covariant cast
  private ImmutableAcquisitionFlow(
      String redirectUrl,
      Type type,
      Optional<? extends CredentialFlowState> state) {
    this.redirectUrl = redirectUrl;
    this.type = type;
    this.state = state.orElse(null);
  }

  private ImmutableAcquisitionFlow(
      ImmutableAcquisitionFlow original,
      String redirectUrl,
      Type type,
      @Nullable CredentialFlowState state) {
    this.redirectUrl = redirectUrl;
    this.type = type;
    this.state = state;
  }

  /**
   * @return The value of the {@code redirectUrl} attribute
   */
  @JsonProperty("redirectUrl")
  @Override
  public String getRedirectUrl() {
    return redirectUrl;
  }

  /**
   * @return The value of the {@code type} attribute
   */
  @JsonProperty("type")
  @Override
  public Type getType() {
    return type;
  }

  /**
   * @return The value of the {@code state} attribute
   */
  @JsonProperty("state")
  @Override
  public Optional<CredentialFlowState> state() {
    return Optional.ofNullable(state);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionFlow#getRedirectUrl() redirectUrl} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for redirectUrl (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionFlow withRedirectUrl(String value) {
    if (Objects.equals(this.redirectUrl, value)) return this;
    return validate(new ImmutableAcquisitionFlow(this, value, this.type, this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionFlow#getType() type} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for type (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionFlow withType(Type value) {
    if (this.type == value) return this;
    if (Objects.equals(this.type, value)) return this;
    return validate(new ImmutableAcquisitionFlow(this, this.redirectUrl, value, this.state));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link AcquisitionFlow#state() state} attribute.
   * @param value The value for state
   * @return A modified copy of {@code this} object
   */
  public final ImmutableAcquisitionFlow withState(CredentialFlowState value) {
    @Nullable CredentialFlowState newValue = Objects.requireNonNull(value, "state");
    if (this.state == newValue) return this;
    return validate(new ImmutableAcquisitionFlow(this, this.redirectUrl, this.type, newValue));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link AcquisitionFlow#state() state} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for state
   * @return A modified copy of {@code this} object
   */
  @SuppressWarnings("unchecked") // safe covariant cast
  public final ImmutableAcquisitionFlow withState(Optional<? extends CredentialFlowState> optional) {
    @Nullable CredentialFlowState value = optional.orElse(null);
    if (this.state == value) return this;
    return validate(new ImmutableAcquisitionFlow(this, this.redirectUrl, this.type, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableAcquisitionFlow} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableAcquisitionFlow
        && equalTo((ImmutableAcquisitionFlow) another);
  }

  private boolean equalTo(ImmutableAcquisitionFlow another) {
    return Objects.equals(redirectUrl, another.redirectUrl)
        && Objects.equals(type, another.type)
        && Objects.equals(state, another.state);
  }

  /**
   * Computes a hash code from attributes: {@code redirectUrl}, {@code type}, {@code state}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(redirectUrl);
    h += (h << 5) + Objects.hashCode(type);
    h += (h << 5) + Objects.hashCode(state);
    return h;
  }

  /**
   * Prints the immutable value {@code AcquisitionFlow} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("AcquisitionFlow{");
    if (redirectUrl != null) {
      builder.append("redirectUrl=").append(redirectUrl);
    }
    if (type != null) {
      if (builder.length() > 16) builder.append(", ");
      builder.append("type=").append(type);
    }
    if (state != null) {
      if (builder.length() > 16) builder.append(", ");
      builder.append("state=").append(state);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code AcquisitionFlow} instance.
   * @param redirectUrl The value for the {@code redirectUrl} attribute
   * @param type The value for the {@code type} attribute
   * @param state The value for the {@code state} attribute
   * @return An immutable AcquisitionFlow instance
   */
  public static AcquisitionFlow of(String redirectUrl, Type type, Optional<? extends CredentialFlowState> state) {
    return validate(new ImmutableAcquisitionFlow(redirectUrl, type, state));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableAcquisitionFlow validate(ImmutableAcquisitionFlow instance) {
    Set<ConstraintViolation<ImmutableAcquisitionFlow>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link AcquisitionFlow} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable AcquisitionFlow instance
   */
  public static AcquisitionFlow copyOf(AcquisitionFlow instance) {
    if (instance instanceof ImmutableAcquisitionFlow) {
      return (ImmutableAcquisitionFlow) instance;
    }
    return new AcquisitionFlow.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link AcquisitionFlow AcquisitionFlow}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "AcquisitionFlow", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private @Nullable String redirectUrl;
    private @Nullable Type type;
    private @Nullable CredentialFlowState state;

    /**
     * Creates a builder for {@link AcquisitionFlow AcquisitionFlow} instances.
     * <pre>
     * new AcquisitionFlow.Builder()
     *    .redirectUrl(String | null) // nullable {@link AcquisitionFlow#getRedirectUrl() redirectUrl}
     *    .type(io.syndesis.server.credential.Type | null) // nullable {@link AcquisitionFlow#getType() type}
     *    .state(io.syndesis.server.credential.CredentialFlowState) // optional {@link AcquisitionFlow#state() state}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof AcquisitionFlow.Builder)) {
        throw new UnsupportedOperationException("Use: new AcquisitionFlow.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code AcquisitionFlow} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final AcquisitionFlow.Builder createFrom(AcquisitionFlow instance) {
      Objects.requireNonNull(instance, "instance");
      String redirectUrlValue = instance.getRedirectUrl();
      if (redirectUrlValue != null) {
        redirectUrl(redirectUrlValue);
      }
      Type typeValue = instance.getType();
      if (typeValue != null) {
        type(typeValue);
      }
      Optional<CredentialFlowState> stateOptional = instance.state();
      if (stateOptional.isPresent()) {
        state(stateOptional);
      }
      return (AcquisitionFlow.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionFlow#getRedirectUrl() redirectUrl} attribute.
     * @param redirectUrl The value for redirectUrl (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("redirectUrl")
    public final AcquisitionFlow.Builder redirectUrl(String redirectUrl) {
      this.redirectUrl = redirectUrl;
      return (AcquisitionFlow.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionFlow#getType() type} attribute.
     * @param type The value for type (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("type")
    public final AcquisitionFlow.Builder type(Type type) {
      this.type = type;
      return (AcquisitionFlow.Builder) this;
    }

    /**
     * Initializes the optional value {@link AcquisitionFlow#state() state} to state.
     * @param state The value for state
     * @return {@code this} builder for chained invocation
     */
    public final AcquisitionFlow.Builder state(CredentialFlowState state) {
      this.state = Objects.requireNonNull(state, "state");
      return (AcquisitionFlow.Builder) this;
    }

    /**
     * Initializes the optional value {@link AcquisitionFlow#state() state} to state.
     * @param state The value for state
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("state")
    public final AcquisitionFlow.Builder state(Optional<? extends CredentialFlowState> state) {
      this.state = state.orElse(null);
      return (AcquisitionFlow.Builder) this;
    }

    /**
     * Builds a new {@link AcquisitionFlow AcquisitionFlow}.
     * @return An immutable instance of AcquisitionFlow
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public AcquisitionFlow build() {
      return ImmutableAcquisitionFlow.validate(new ImmutableAcquisitionFlow(null, redirectUrl, type, state));
    }
  }
}
