/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.credential;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link AcquisitionMethod}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new AcquisitionMethod.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableAcquisitionMethod.of()}.
 */
@Generated(from = "AcquisitionMethod", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableAcquisitionMethod implements AcquisitionMethod {
  private final String description;
  private final String icon;
  private final String label;
  private final Type type;

  private ImmutableAcquisitionMethod(
      String description,
      String icon,
      String label,
      Type type) {
    this.description = description;
    this.icon = icon;
    this.label = label;
    this.type = type;
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @JsonProperty("description")
  @Override
  public String getDescription() {
    return description;
  }

  /**
   * @return The value of the {@code icon} attribute
   */
  @JsonProperty("icon")
  @Override
  public String getIcon() {
    return icon;
  }

  /**
   * @return The value of the {@code label} attribute
   */
  @JsonProperty("label")
  @Override
  public String getLabel() {
    return label;
  }

  /**
   * @return The value of the {@code type} attribute
   */
  @JsonProperty("type")
  @Override
  public Type getType() {
    return type;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionMethod#getDescription() description} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for description (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionMethod withDescription(String value) {
    if (Objects.equals(this.description, value)) return this;
    return validate(new ImmutableAcquisitionMethod(value, this.icon, this.label, this.type));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionMethod#getIcon() icon} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for icon (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionMethod withIcon(String value) {
    if (Objects.equals(this.icon, value)) return this;
    return validate(new ImmutableAcquisitionMethod(this.description, value, this.label, this.type));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionMethod#getLabel() label} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for label (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionMethod withLabel(String value) {
    if (Objects.equals(this.label, value)) return this;
    return validate(new ImmutableAcquisitionMethod(this.description, this.icon, value, this.type));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionMethod#getType() type} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for type (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionMethod withType(Type value) {
    if (this.type == value) return this;
    if (Objects.equals(this.type, value)) return this;
    return validate(new ImmutableAcquisitionMethod(this.description, this.icon, this.label, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableAcquisitionMethod} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableAcquisitionMethod
        && equalTo((ImmutableAcquisitionMethod) another);
  }

  private boolean equalTo(ImmutableAcquisitionMethod another) {
    return Objects.equals(description, another.description)
        && Objects.equals(icon, another.icon)
        && Objects.equals(label, another.label)
        && Objects.equals(type, another.type);
  }

  /**
   * Computes a hash code from attributes: {@code description}, {@code icon}, {@code label}, {@code type}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(description);
    h += (h << 5) + Objects.hashCode(icon);
    h += (h << 5) + Objects.hashCode(label);
    h += (h << 5) + Objects.hashCode(type);
    return h;
  }

  /**
   * Prints the immutable value {@code AcquisitionMethod} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "AcquisitionMethod{"
        + "description=" + description
        + ", icon=" + icon
        + ", label=" + label
        + ", type=" + type
        + "}";
  }

  /**
   * Construct a new immutable {@code AcquisitionMethod} instance.
   * @param description The value for the {@code description} attribute
   * @param icon The value for the {@code icon} attribute
   * @param label The value for the {@code label} attribute
   * @param type The value for the {@code type} attribute
   * @return An immutable AcquisitionMethod instance
   */
  public static AcquisitionMethod of(String description, String icon, String label, Type type) {
    return validate(new ImmutableAcquisitionMethod(description, icon, label, type));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableAcquisitionMethod validate(ImmutableAcquisitionMethod instance) {
    Set<ConstraintViolation<ImmutableAcquisitionMethod>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link AcquisitionMethod} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable AcquisitionMethod instance
   */
  public static AcquisitionMethod copyOf(AcquisitionMethod instance) {
    if (instance instanceof ImmutableAcquisitionMethod) {
      return (ImmutableAcquisitionMethod) instance;
    }
    return new AcquisitionMethod.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link AcquisitionMethod AcquisitionMethod}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "AcquisitionMethod", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private @Nullable String description;
    private @Nullable String icon;
    private @Nullable String label;
    private @Nullable Type type;

    /**
     * Creates a builder for {@link AcquisitionMethod AcquisitionMethod} instances.
     * <pre>
     * new AcquisitionMethod.Builder()
     *    .description(String | null) // nullable {@link AcquisitionMethod#getDescription() description}
     *    .icon(String | null) // nullable {@link AcquisitionMethod#getIcon() icon}
     *    .label(String | null) // nullable {@link AcquisitionMethod#getLabel() label}
     *    .type(io.syndesis.server.credential.Type | null) // nullable {@link AcquisitionMethod#getType() type}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof AcquisitionMethod.Builder)) {
        throw new UnsupportedOperationException("Use: new AcquisitionMethod.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code AcquisitionMethod} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final AcquisitionMethod.Builder createFrom(AcquisitionMethod instance) {
      Objects.requireNonNull(instance, "instance");
      String descriptionValue = instance.getDescription();
      if (descriptionValue != null) {
        description(descriptionValue);
      }
      String iconValue = instance.getIcon();
      if (iconValue != null) {
        icon(iconValue);
      }
      String labelValue = instance.getLabel();
      if (labelValue != null) {
        label(labelValue);
      }
      Type typeValue = instance.getType();
      if (typeValue != null) {
        type(typeValue);
      }
      return (AcquisitionMethod.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionMethod#getDescription() description} attribute.
     * @param description The value for description (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("description")
    public final AcquisitionMethod.Builder description(String description) {
      this.description = description;
      return (AcquisitionMethod.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionMethod#getIcon() icon} attribute.
     * @param icon The value for icon (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("icon")
    public final AcquisitionMethod.Builder icon(String icon) {
      this.icon = icon;
      return (AcquisitionMethod.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionMethod#getLabel() label} attribute.
     * @param label The value for label (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("label")
    public final AcquisitionMethod.Builder label(String label) {
      this.label = label;
      return (AcquisitionMethod.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionMethod#getType() type} attribute.
     * @param type The value for type (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("type")
    public final AcquisitionMethod.Builder type(Type type) {
      this.type = type;
      return (AcquisitionMethod.Builder) this;
    }

    /**
     * Builds a new {@link AcquisitionMethod AcquisitionMethod}.
     * @return An immutable instance of AcquisitionMethod
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public AcquisitionMethod build() {
      return ImmutableAcquisitionMethod.validate(new ImmutableAcquisitionMethod(description, icon, label, type));
    }
  }
}
