package io.syndesis.server.credential;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link AcquisitionRequest}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new AcquisitionRequest.Builder()}.
 */
@Generated(from = "AcquisitionRequest", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
public final class ImmutableAcquisitionRequest implements AcquisitionRequest {
  private final URI returnUrl;

  private ImmutableAcquisitionRequest(URI returnUrl) {
    this.returnUrl = returnUrl;
  }

  /**
   * @return The value of the {@code returnUrl} attribute
   */
  @JsonProperty("returnUrl")
  @RelativeUri
  @Override
  public URI getReturnUrl() {
    return returnUrl;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionRequest#getReturnUrl() returnUrl} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for returnUrl
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionRequest withReturnUrl(URI value) {
    if (this.returnUrl == value) return this;
    URI newValue = Objects.requireNonNull(value, "returnUrl");
    return new ImmutableAcquisitionRequest(newValue);
  }

  /**
   * This instance is equal to all instances of {@code ImmutableAcquisitionRequest} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableAcquisitionRequest
        && equalTo((ImmutableAcquisitionRequest) another);
  }

  private boolean equalTo(ImmutableAcquisitionRequest another) {
    return returnUrl.equals(another.returnUrl);
  }

  /**
   * Computes a hash code from attributes: {@code returnUrl}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + returnUrl.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code AcquisitionRequest} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "AcquisitionRequest{"
        + "returnUrl=" + returnUrl
        + "}";
  }

  /**
   * Creates an immutable copy of a {@link AcquisitionRequest} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable AcquisitionRequest instance
   */
  public static ImmutableAcquisitionRequest copyOf(AcquisitionRequest instance) {
    if (instance instanceof ImmutableAcquisitionRequest) {
      return (ImmutableAcquisitionRequest) instance;
    }
    return new AcquisitionRequest.Builder()
        .from(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableAcquisitionRequest ImmutableAcquisitionRequest}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "AcquisitionRequest", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private static final long INIT_BIT_RETURN_URL = 0x1L;
    private long initBits = 0x1L;

    private @Nullable URI returnUrl;

    /**
     * Creates a builder for {@link ImmutableAcquisitionRequest ImmutableAcquisitionRequest} instances.
     * <pre>
     * new AcquisitionRequest.Builder()
     *    .returnUrl(java.net.URI) // required {@link AcquisitionRequest#getReturnUrl() returnUrl}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof AcquisitionRequest.Builder)) {
        throw new UnsupportedOperationException("Use: new AcquisitionRequest.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code AcquisitionRequest} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final AcquisitionRequest.Builder from(AcquisitionRequest instance) {
      Objects.requireNonNull(instance, "instance");
      returnUrl(instance.getReturnUrl());
      return (AcquisitionRequest.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionRequest#getReturnUrl() returnUrl} attribute.
     * @param returnUrl The value for returnUrl 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("returnUrl")
    public final AcquisitionRequest.Builder returnUrl(URI returnUrl) {
      this.returnUrl = Objects.requireNonNull(returnUrl, "returnUrl");
      initBits &= ~INIT_BIT_RETURN_URL;
      return (AcquisitionRequest.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableAcquisitionRequest ImmutableAcquisitionRequest}.
     * @return An immutable instance of AcquisitionRequest
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableAcquisitionRequest build() {
      if (initBits != 0) {
        throw new IllegalStateException(formatRequiredAttributesMessage());
      }
      return new ImmutableAcquisitionRequest(returnUrl);
    }

    private String formatRequiredAttributesMessage() {
      List<String> attributes = new ArrayList<>();
      if ((initBits & INIT_BIT_RETURN_URL) != 0) attributes.add("returnUrl");
      return "Cannot build AcquisitionRequest, some of required attributes are not set " + attributes;
    }
  }
}
