/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.credential;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link AcquisitionResponse}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new AcquisitionResponse.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableAcquisitionResponse.of()}.
 */
@Generated(from = "AcquisitionResponse", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableAcquisitionResponse implements AcquisitionResponse {
  private final String redirectUrl;
  private final Type type;
  private final AcquisitionResponse.State state;

  private ImmutableAcquisitionResponse(
      String redirectUrl,
      Type type,
      AcquisitionResponse.State state) {
    this.redirectUrl = redirectUrl;
    this.type = type;
    this.state = state;
  }

  /**
   * @return The value of the {@code redirectUrl} attribute
   */
  @JsonProperty("redirectUrl")
  @Override
  public String getRedirectUrl() {
    return redirectUrl;
  }

  /**
   * @return The value of the {@code type} attribute
   */
  @JsonProperty("type")
  @Override
  public Type getType() {
    return type;
  }

  /**
   * @return The value of the {@code state} attribute
   */
  @JsonProperty("state")
  @Override
  public AcquisitionResponse.State state() {
    return state;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionResponse#getRedirectUrl() redirectUrl} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for redirectUrl (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionResponse withRedirectUrl(String value) {
    if (Objects.equals(this.redirectUrl, value)) return this;
    return validate(new ImmutableAcquisitionResponse(value, this.type, this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionResponse#getType() type} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for type (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionResponse withType(Type value) {
    if (this.type == value) return this;
    if (Objects.equals(this.type, value)) return this;
    return validate(new ImmutableAcquisitionResponse(this.redirectUrl, value, this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionResponse#state() state} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for state (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAcquisitionResponse withState(AcquisitionResponse.State value) {
    if (this.state == value) return this;
    return validate(new ImmutableAcquisitionResponse(this.redirectUrl, this.type, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableAcquisitionResponse} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableAcquisitionResponse
        && equalTo((ImmutableAcquisitionResponse) another);
  }

  private boolean equalTo(ImmutableAcquisitionResponse another) {
    return Objects.equals(redirectUrl, another.redirectUrl)
        && Objects.equals(type, another.type)
        && Objects.equals(state, another.state);
  }

  /**
   * Computes a hash code from attributes: {@code redirectUrl}, {@code type}, {@code state}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(redirectUrl);
    h += (h << 5) + Objects.hashCode(type);
    h += (h << 5) + Objects.hashCode(state);
    return h;
  }

  /**
   * Prints the immutable value {@code AcquisitionResponse} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "AcquisitionResponse{"
        + "redirectUrl=" + redirectUrl
        + ", type=" + type
        + ", state=" + state
        + "}";
  }

  /**
   * Construct a new immutable {@code AcquisitionResponse} instance.
   * @param redirectUrl The value for the {@code redirectUrl} attribute
   * @param type The value for the {@code type} attribute
   * @param state The value for the {@code state} attribute
   * @return An immutable AcquisitionResponse instance
   */
  public static AcquisitionResponse of(String redirectUrl, Type type, AcquisitionResponse.State state) {
    return validate(new ImmutableAcquisitionResponse(redirectUrl, type, state));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableAcquisitionResponse validate(ImmutableAcquisitionResponse instance) {
    Set<ConstraintViolation<ImmutableAcquisitionResponse>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link AcquisitionResponse} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable AcquisitionResponse instance
   */
  public static AcquisitionResponse copyOf(AcquisitionResponse instance) {
    if (instance instanceof ImmutableAcquisitionResponse) {
      return (ImmutableAcquisitionResponse) instance;
    }
    return new AcquisitionResponse.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link AcquisitionResponse AcquisitionResponse}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "AcquisitionResponse", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private @Nullable String redirectUrl;
    private @Nullable Type type;
    private @Nullable AcquisitionResponse.State state;

    /**
     * Creates a builder for {@link AcquisitionResponse AcquisitionResponse} instances.
     * <pre>
     * new AcquisitionResponse.Builder()
     *    .redirectUrl(String | null) // nullable {@link AcquisitionResponse#getRedirectUrl() redirectUrl}
     *    .type(io.syndesis.server.credential.Type | null) // nullable {@link AcquisitionResponse#getType() type}
     *    .state(io.syndesis.server.credential.AcquisitionResponse.State | null) // nullable {@link AcquisitionResponse#state() state}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof AcquisitionResponse.Builder)) {
        throw new UnsupportedOperationException("Use: new AcquisitionResponse.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code AcquisitionResponse} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final AcquisitionResponse.Builder createFrom(AcquisitionResponse instance) {
      Objects.requireNonNull(instance, "instance");
      String redirectUrlValue = instance.getRedirectUrl();
      if (redirectUrlValue != null) {
        redirectUrl(redirectUrlValue);
      }
      Type typeValue = instance.getType();
      if (typeValue != null) {
        type(typeValue);
      }
      AcquisitionResponse.State stateValue = instance.state();
      if (stateValue != null) {
        state(stateValue);
      }
      return (AcquisitionResponse.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionResponse#getRedirectUrl() redirectUrl} attribute.
     * @param redirectUrl The value for redirectUrl (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("redirectUrl")
    public final AcquisitionResponse.Builder redirectUrl(String redirectUrl) {
      this.redirectUrl = redirectUrl;
      return (AcquisitionResponse.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionResponse#getType() type} attribute.
     * @param type The value for type (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("type")
    public final AcquisitionResponse.Builder type(Type type) {
      this.type = type;
      return (AcquisitionResponse.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionResponse#state() state} attribute.
     * @param state The value for state (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("state")
    public final AcquisitionResponse.Builder state(AcquisitionResponse.State state) {
      this.state = state;
      return (AcquisitionResponse.Builder) this;
    }

    /**
     * Builds a new {@link AcquisitionResponse AcquisitionResponse}.
     * @return An immutable instance of AcquisitionResponse
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public AcquisitionResponse build() {
      return ImmutableAcquisitionResponse.validate(new ImmutableAcquisitionResponse(redirectUrl, type, state));
    }
  }
}
