/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.credential;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.net.URI;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;
import org.springframework.social.oauth1.OAuthToken;

/**
 * Immutable implementation of {@link OAuth1CredentialFlowState}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new OAuth1CredentialFlowState.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableOAuth1CredentialFlowState.of()}.
 */
@Generated(from = "OAuth1CredentialFlowState", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableOAuth1CredentialFlowState
    implements OAuth1CredentialFlowState {
  private final String connectorId;
  private final String key;
  private final String providerId;
  private final String redirectUrl;
  private final URI returnUrl;
  private final OAuthToken accessToken;
  private final OAuthToken token;
  private final String verifier;

  private ImmutableOAuth1CredentialFlowState(
      String connectorId,
      String key,
      String providerId,
      String redirectUrl,
      URI returnUrl,
      OAuthToken accessToken,
      OAuthToken token,
      String verifier) {
    this.connectorId = Objects.requireNonNull(connectorId, "connectorId");
    this.key = key;
    this.providerId = providerId;
    this.redirectUrl = redirectUrl;
    this.returnUrl = returnUrl;
    this.accessToken = accessToken;
    this.token = token;
    this.verifier = verifier;
  }

  private ImmutableOAuth1CredentialFlowState(ImmutableOAuth1CredentialFlowState.Builder builder) {
    this.key = builder.key;
    this.providerId = builder.providerId;
    this.redirectUrl = builder.redirectUrl;
    this.returnUrl = builder.returnUrl;
    this.accessToken = builder.accessToken;
    this.token = builder.token;
    this.verifier = builder.verifier;
    this.connectorId = builder.connectorId != null
        ? builder.connectorId
        : Objects.requireNonNull(OAuth1CredentialFlowState.super.getConnectorId(), "connectorId");
  }

  private ImmutableOAuth1CredentialFlowState(
      ImmutableOAuth1CredentialFlowState original,
      String connectorId,
      String key,
      String providerId,
      String redirectUrl,
      URI returnUrl,
      OAuthToken accessToken,
      OAuthToken token,
      String verifier) {
    this.connectorId = connectorId;
    this.key = key;
    this.providerId = providerId;
    this.redirectUrl = redirectUrl;
    this.returnUrl = returnUrl;
    this.accessToken = accessToken;
    this.token = token;
    this.verifier = verifier;
  }

  /**
   * @return The value of the {@code connectorId} attribute
   */
  @JsonProperty("connectorId")
  @Override
  public String getConnectorId() {
    return connectorId;
  }

  /**
   * @return The value of the {@code key} attribute
   */
  @JsonProperty("key")
  @Override
  public String getKey() {
    return key;
  }

  /**
   * @return The value of the {@code providerId} attribute
   */
  @JsonProperty("providerId")
  @Override
  public String getProviderId() {
    return providerId;
  }

  /**
   * @return The value of the {@code redirectUrl} attribute
   */
  @JsonProperty("redirectUrl")
  @JsonIgnore
  @Override
  public String getRedirectUrl() {
    return redirectUrl;
  }

  /**
   * @return The value of the {@code returnUrl} attribute
   */
  @JsonProperty("returnUrl")
  @Override
  public URI getReturnUrl() {
    return returnUrl;
  }

  /**
   * @return The value of the {@code accessToken} attribute
   */
  @JsonProperty("accessToken")
  @Override
  public OAuthToken getAccessToken() {
    return accessToken;
  }

  /**
   * @return The value of the {@code token} attribute
   */
  @JsonProperty("token")
  @JsonDeserialize(converter = OAuth1CredentialFlowState.OAuthTokenConverter.class)
  @Override
  public OAuthToken getToken() {
    return token;
  }

  /**
   * @return The value of the {@code verifier} attribute
   */
  @JsonProperty("verifier")
  @Override
  public String getVerifier() {
    return verifier;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth1CredentialFlowState#getConnectorId() connectorId} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for connectorId
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth1CredentialFlowState withConnectorId(String value) {
    String newValue = Objects.requireNonNull(value, "connectorId");
    if (this.connectorId.equals(newValue)) return this;
    return validate(new ImmutableOAuth1CredentialFlowState(
        this,
        newValue,
        this.key,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        this.accessToken,
        this.token,
        this.verifier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth1CredentialFlowState#getKey() key} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for key (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth1CredentialFlowState withKey(String value) {
    if (Objects.equals(this.key, value)) return this;
    return validate(new ImmutableOAuth1CredentialFlowState(
        this,
        this.connectorId,
        value,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        this.accessToken,
        this.token,
        this.verifier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth1CredentialFlowState#getProviderId() providerId} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for providerId (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth1CredentialFlowState withProviderId(String value) {
    if (Objects.equals(this.providerId, value)) return this;
    return validate(new ImmutableOAuth1CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        value,
        this.redirectUrl,
        this.returnUrl,
        this.accessToken,
        this.token,
        this.verifier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth1CredentialFlowState#getRedirectUrl() redirectUrl} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for redirectUrl (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth1CredentialFlowState withRedirectUrl(String value) {
    if (Objects.equals(this.redirectUrl, value)) return this;
    return validate(new ImmutableOAuth1CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        value,
        this.returnUrl,
        this.accessToken,
        this.token,
        this.verifier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth1CredentialFlowState#getReturnUrl() returnUrl} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for returnUrl (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth1CredentialFlowState withReturnUrl(URI value) {
    if (this.returnUrl == value) return this;
    return validate(new ImmutableOAuth1CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        this.redirectUrl,
        value,
        this.accessToken,
        this.token,
        this.verifier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth1CredentialFlowState#getAccessToken() accessToken} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for accessToken (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth1CredentialFlowState withAccessToken(OAuthToken value) {
    if (this.accessToken == value) return this;
    return validate(new ImmutableOAuth1CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        value,
        this.token,
        this.verifier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth1CredentialFlowState#getToken() token} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for token (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth1CredentialFlowState withToken(OAuthToken value) {
    if (this.token == value) return this;
    return validate(new ImmutableOAuth1CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        this.accessToken,
        value,
        this.verifier));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth1CredentialFlowState#getVerifier() verifier} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for verifier (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth1CredentialFlowState withVerifier(String value) {
    if (Objects.equals(this.verifier, value)) return this;
    return validate(new ImmutableOAuth1CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        this.accessToken,
        this.token,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableOAuth1CredentialFlowState} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableOAuth1CredentialFlowState
        && equalTo((ImmutableOAuth1CredentialFlowState) another);
  }

  private boolean equalTo(ImmutableOAuth1CredentialFlowState another) {
    return connectorId.equals(another.connectorId)
        && Objects.equals(key, another.key)
        && Objects.equals(providerId, another.providerId)
        && Objects.equals(redirectUrl, another.redirectUrl)
        && Objects.equals(returnUrl, another.returnUrl)
        && Objects.equals(accessToken, another.accessToken)
        && Objects.equals(token, another.token)
        && Objects.equals(verifier, another.verifier);
  }

  /**
   * Computes a hash code from attributes: {@code connectorId}, {@code key}, {@code providerId}, {@code redirectUrl}, {@code returnUrl}, {@code accessToken}, {@code token}, {@code verifier}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + connectorId.hashCode();
    h += (h << 5) + Objects.hashCode(key);
    h += (h << 5) + Objects.hashCode(providerId);
    h += (h << 5) + Objects.hashCode(redirectUrl);
    h += (h << 5) + Objects.hashCode(returnUrl);
    h += (h << 5) + Objects.hashCode(accessToken);
    h += (h << 5) + Objects.hashCode(token);
    h += (h << 5) + Objects.hashCode(verifier);
    return h;
  }

  /**
   * Prints the immutable value {@code OAuth1CredentialFlowState} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "OAuth1CredentialFlowState{"
        + "connectorId=" + connectorId
        + ", key=" + key
        + ", providerId=" + providerId
        + ", redirectUrl=" + redirectUrl
        + ", returnUrl=" + returnUrl
        + ", accessToken=" + accessToken
        + ", token=" + token
        + ", verifier=" + verifier
        + "}";
  }

  /**
   * Construct a new immutable {@code OAuth1CredentialFlowState} instance.
   * @param connectorId The value for the {@code connectorId} attribute
   * @param key The value for the {@code key} attribute
   * @param providerId The value for the {@code providerId} attribute
   * @param redirectUrl The value for the {@code redirectUrl} attribute
   * @param returnUrl The value for the {@code returnUrl} attribute
   * @param accessToken The value for the {@code accessToken} attribute
   * @param token The value for the {@code token} attribute
   * @param verifier The value for the {@code verifier} attribute
   * @return An immutable OAuth1CredentialFlowState instance
   */
  public static OAuth1CredentialFlowState of(String connectorId, String key, String providerId, String redirectUrl, URI returnUrl, OAuthToken accessToken, OAuthToken token, String verifier) {
    return validate(new ImmutableOAuth1CredentialFlowState(connectorId, key, providerId, redirectUrl, returnUrl, accessToken, token, verifier));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableOAuth1CredentialFlowState validate(ImmutableOAuth1CredentialFlowState instance) {
    Set<ConstraintViolation<ImmutableOAuth1CredentialFlowState>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link OAuth1CredentialFlowState} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable OAuth1CredentialFlowState instance
   */
  public static OAuth1CredentialFlowState copyOf(OAuth1CredentialFlowState instance) {
    if (instance instanceof ImmutableOAuth1CredentialFlowState) {
      return (ImmutableOAuth1CredentialFlowState) instance;
    }
    return new OAuth1CredentialFlowState.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link OAuth1CredentialFlowState OAuth1CredentialFlowState}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "OAuth1CredentialFlowState", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private @Nullable String connectorId;
    private @Nullable String key;
    private @Nullable String providerId;
    private @Nullable String redirectUrl;
    private @Nullable URI returnUrl;
    private @Nullable OAuthToken accessToken;
    private @Nullable OAuthToken token;
    private @Nullable String verifier;

    /**
     * Creates a builder for {@link OAuth1CredentialFlowState OAuth1CredentialFlowState} instances.
     * <pre>
     * new OAuth1CredentialFlowState.Builder()
     *    .connectorId(String) // optional {@link OAuth1CredentialFlowState#getConnectorId() connectorId}
     *    .key(String | null) // nullable {@link OAuth1CredentialFlowState#getKey() key}
     *    .providerId(String | null) // nullable {@link OAuth1CredentialFlowState#getProviderId() providerId}
     *    .redirectUrl(String | null) // nullable {@link OAuth1CredentialFlowState#getRedirectUrl() redirectUrl}
     *    .returnUrl(java.net.URI | null) // nullable {@link OAuth1CredentialFlowState#getReturnUrl() returnUrl}
     *    .accessToken(org.springframework.social.oauth1.OAuthToken | null) // nullable {@link OAuth1CredentialFlowState#getAccessToken() accessToken}
     *    .token(org.springframework.social.oauth1.OAuthToken | null) // nullable {@link OAuth1CredentialFlowState#getToken() token}
     *    .verifier(String | null) // nullable {@link OAuth1CredentialFlowState#getVerifier() verifier}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof OAuth1CredentialFlowState.Builder)) {
        throw new UnsupportedOperationException("Use: new OAuth1CredentialFlowState.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.server.credential.CredentialFlowState} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final OAuth1CredentialFlowState.Builder createFrom(CredentialFlowState instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.server.credential.OAuth1CredentialFlowState} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final OAuth1CredentialFlowState.Builder createFrom(OAuth1CredentialFlowState instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuth1CredentialFlowState.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof CredentialFlowState) {
        CredentialFlowState instance = (CredentialFlowState) object;
        String redirectUrlValue = instance.getRedirectUrl();
        if (redirectUrlValue != null) {
          redirectUrl(redirectUrlValue);
        }
        URI returnUrlValue = instance.getReturnUrl();
        if (returnUrlValue != null) {
          returnUrl(returnUrlValue);
        }
        connectorId(instance.getConnectorId());
        String providerIdValue = instance.getProviderId();
        if (providerIdValue != null) {
          providerId(providerIdValue);
        }
        String keyValue = instance.getKey();
        if (keyValue != null) {
          key(keyValue);
        }
      }
      if (object instanceof OAuth1CredentialFlowState) {
        OAuth1CredentialFlowState instance = (OAuth1CredentialFlowState) object;
        OAuthToken accessTokenValue = instance.getAccessToken();
        if (accessTokenValue != null) {
          accessToken(accessTokenValue);
        }
        String verifierValue = instance.getVerifier();
        if (verifierValue != null) {
          verifier(verifierValue);
        }
        OAuthToken tokenValue = instance.getToken();
        if (tokenValue != null) {
          token(tokenValue);
        }
      }
    }

    /**
     * Initializes the value for the {@link OAuth1CredentialFlowState#getConnectorId() connectorId} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link OAuth1CredentialFlowState#getConnectorId() connectorId}.</em>
     * @param connectorId The value for connectorId 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connectorId")
    public final OAuth1CredentialFlowState.Builder connectorId(String connectorId) {
      this.connectorId = Objects.requireNonNull(connectorId, "connectorId");
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth1CredentialFlowState#getKey() key} attribute.
     * @param key The value for key (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("key")
    public final OAuth1CredentialFlowState.Builder key(String key) {
      this.key = key;
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth1CredentialFlowState#getProviderId() providerId} attribute.
     * @param providerId The value for providerId (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("providerId")
    public final OAuth1CredentialFlowState.Builder providerId(String providerId) {
      this.providerId = providerId;
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth1CredentialFlowState#getRedirectUrl() redirectUrl} attribute.
     * @param redirectUrl The value for redirectUrl (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("redirectUrl")
    @JsonIgnore
    public final OAuth1CredentialFlowState.Builder redirectUrl(String redirectUrl) {
      this.redirectUrl = redirectUrl;
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth1CredentialFlowState#getReturnUrl() returnUrl} attribute.
     * @param returnUrl The value for returnUrl (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("returnUrl")
    public final OAuth1CredentialFlowState.Builder returnUrl(URI returnUrl) {
      this.returnUrl = returnUrl;
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth1CredentialFlowState#getAccessToken() accessToken} attribute.
     * @param accessToken The value for accessToken (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("accessToken")
    public final OAuth1CredentialFlowState.Builder accessToken(OAuthToken accessToken) {
      this.accessToken = accessToken;
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth1CredentialFlowState#getToken() token} attribute.
     * @param token The value for token (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("token")
    @JsonDeserialize(converter = OAuth1CredentialFlowState.OAuthTokenConverter.class)
    public final OAuth1CredentialFlowState.Builder token(OAuthToken token) {
      this.token = token;
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth1CredentialFlowState#getVerifier() verifier} attribute.
     * @param verifier The value for verifier (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("verifier")
    public final OAuth1CredentialFlowState.Builder verifier(String verifier) {
      this.verifier = verifier;
      return (OAuth1CredentialFlowState.Builder) this;
    }

    /**
     * Builds a new {@link OAuth1CredentialFlowState OAuth1CredentialFlowState}.
     * @return An immutable instance of OAuth1CredentialFlowState
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public OAuth1CredentialFlowState build() {
      return ImmutableOAuth1CredentialFlowState.validate(new ImmutableOAuth1CredentialFlowState(this));
    }
  }
}
