/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.credential;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.net.URI;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;
import org.springframework.social.oauth2.AccessGrant;

/**
 * Immutable implementation of {@link OAuth2CredentialFlowState}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new OAuth2CredentialFlowState.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableOAuth2CredentialFlowState.of()}.
 */
@Generated(from = "OAuth2CredentialFlowState", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableOAuth2CredentialFlowState
    implements OAuth2CredentialFlowState {
  private final String connectorId;
  private final String key;
  private final String providerId;
  private final String redirectUrl;
  private final URI returnUrl;
  private final AccessGrant accessGrant;
  private final String code;
  private final String state;

  private ImmutableOAuth2CredentialFlowState(
      String connectorId,
      String key,
      String providerId,
      String redirectUrl,
      URI returnUrl,
      AccessGrant accessGrant,
      String code,
      String state) {
    this.connectorId = Objects.requireNonNull(connectorId, "connectorId");
    this.key = key;
    this.providerId = providerId;
    this.redirectUrl = redirectUrl;
    this.returnUrl = returnUrl;
    this.accessGrant = accessGrant;
    this.code = code;
    this.state = state;
  }

  private ImmutableOAuth2CredentialFlowState(ImmutableOAuth2CredentialFlowState.Builder builder) {
    this.key = builder.key;
    this.providerId = builder.providerId;
    this.redirectUrl = builder.redirectUrl;
    this.returnUrl = builder.returnUrl;
    this.accessGrant = builder.accessGrant;
    this.code = builder.code;
    this.state = builder.state;
    this.connectorId = builder.connectorId != null
        ? builder.connectorId
        : Objects.requireNonNull(OAuth2CredentialFlowState.super.getConnectorId(), "connectorId");
  }

  private ImmutableOAuth2CredentialFlowState(
      ImmutableOAuth2CredentialFlowState original,
      String connectorId,
      String key,
      String providerId,
      String redirectUrl,
      URI returnUrl,
      AccessGrant accessGrant,
      String code,
      String state) {
    this.connectorId = connectorId;
    this.key = key;
    this.providerId = providerId;
    this.redirectUrl = redirectUrl;
    this.returnUrl = returnUrl;
    this.accessGrant = accessGrant;
    this.code = code;
    this.state = state;
  }

  /**
   * @return The value of the {@code connectorId} attribute
   */
  @JsonProperty("connectorId")
  @Override
  public String getConnectorId() {
    return connectorId;
  }

  /**
   * @return The value of the {@code key} attribute
   */
  @JsonProperty("key")
  @Override
  public String getKey() {
    return key;
  }

  /**
   * @return The value of the {@code providerId} attribute
   */
  @JsonProperty("providerId")
  @Override
  public String getProviderId() {
    return providerId;
  }

  /**
   * @return The value of the {@code redirectUrl} attribute
   */
  @JsonProperty("redirectUrl")
  @JsonIgnore
  @Override
  public String getRedirectUrl() {
    return redirectUrl;
  }

  /**
   * @return The value of the {@code returnUrl} attribute
   */
  @JsonProperty("returnUrl")
  @Override
  public URI getReturnUrl() {
    return returnUrl;
  }

  /**
   * @return The value of the {@code accessGrant} attribute
   */
  @JsonProperty("accessGrant")
  @JsonDeserialize(converter = OAuth2CredentialFlowState.AccessGrantConverter.class)
  @Override
  public AccessGrant getAccessGrant() {
    return accessGrant;
  }

  /**
   * @return The value of the {@code code} attribute
   */
  @JsonProperty("code")
  @Override
  public String getCode() {
    return code;
  }

  /**
   * @return The value of the {@code state} attribute
   */
  @JsonProperty("state")
  @Override
  public String getState() {
    return state;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth2CredentialFlowState#getConnectorId() connectorId} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for connectorId
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth2CredentialFlowState withConnectorId(String value) {
    String newValue = Objects.requireNonNull(value, "connectorId");
    if (this.connectorId.equals(newValue)) return this;
    return validate(new ImmutableOAuth2CredentialFlowState(
        this,
        newValue,
        this.key,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        this.accessGrant,
        this.code,
        this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth2CredentialFlowState#getKey() key} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for key (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth2CredentialFlowState withKey(String value) {
    if (Objects.equals(this.key, value)) return this;
    return validate(new ImmutableOAuth2CredentialFlowState(
        this,
        this.connectorId,
        value,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        this.accessGrant,
        this.code,
        this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth2CredentialFlowState#getProviderId() providerId} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for providerId (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth2CredentialFlowState withProviderId(String value) {
    if (Objects.equals(this.providerId, value)) return this;
    return validate(new ImmutableOAuth2CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        value,
        this.redirectUrl,
        this.returnUrl,
        this.accessGrant,
        this.code,
        this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth2CredentialFlowState#getRedirectUrl() redirectUrl} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for redirectUrl (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth2CredentialFlowState withRedirectUrl(String value) {
    if (Objects.equals(this.redirectUrl, value)) return this;
    return validate(new ImmutableOAuth2CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        value,
        this.returnUrl,
        this.accessGrant,
        this.code,
        this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth2CredentialFlowState#getReturnUrl() returnUrl} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for returnUrl (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth2CredentialFlowState withReturnUrl(URI value) {
    if (this.returnUrl == value) return this;
    return validate(new ImmutableOAuth2CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        this.redirectUrl,
        value,
        this.accessGrant,
        this.code,
        this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth2CredentialFlowState#getAccessGrant() accessGrant} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for accessGrant (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth2CredentialFlowState withAccessGrant(AccessGrant value) {
    if (this.accessGrant == value) return this;
    return validate(new ImmutableOAuth2CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        value,
        this.code,
        this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth2CredentialFlowState#getCode() code} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for code (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth2CredentialFlowState withCode(String value) {
    if (Objects.equals(this.code, value)) return this;
    return validate(new ImmutableOAuth2CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        this.accessGrant,
        value,
        this.state));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuth2CredentialFlowState#getState() state} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for state (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuth2CredentialFlowState withState(String value) {
    if (Objects.equals(this.state, value)) return this;
    return validate(new ImmutableOAuth2CredentialFlowState(
        this,
        this.connectorId,
        this.key,
        this.providerId,
        this.redirectUrl,
        this.returnUrl,
        this.accessGrant,
        this.code,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableOAuth2CredentialFlowState} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableOAuth2CredentialFlowState
        && equalTo((ImmutableOAuth2CredentialFlowState) another);
  }

  private boolean equalTo(ImmutableOAuth2CredentialFlowState another) {
    return connectorId.equals(another.connectorId)
        && Objects.equals(key, another.key)
        && Objects.equals(providerId, another.providerId)
        && Objects.equals(redirectUrl, another.redirectUrl)
        && Objects.equals(returnUrl, another.returnUrl)
        && Objects.equals(accessGrant, another.accessGrant)
        && Objects.equals(code, another.code)
        && Objects.equals(state, another.state);
  }

  /**
   * Computes a hash code from attributes: {@code connectorId}, {@code key}, {@code providerId}, {@code redirectUrl}, {@code returnUrl}, {@code accessGrant}, {@code code}, {@code state}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + connectorId.hashCode();
    h += (h << 5) + Objects.hashCode(key);
    h += (h << 5) + Objects.hashCode(providerId);
    h += (h << 5) + Objects.hashCode(redirectUrl);
    h += (h << 5) + Objects.hashCode(returnUrl);
    h += (h << 5) + Objects.hashCode(accessGrant);
    h += (h << 5) + Objects.hashCode(code);
    h += (h << 5) + Objects.hashCode(state);
    return h;
  }

  /**
   * Prints the immutable value {@code OAuth2CredentialFlowState} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "OAuth2CredentialFlowState{"
        + "connectorId=" + connectorId
        + ", key=" + key
        + ", providerId=" + providerId
        + ", redirectUrl=" + redirectUrl
        + ", returnUrl=" + returnUrl
        + ", accessGrant=" + accessGrant
        + ", code=" + code
        + ", state=" + state
        + "}";
  }

  /**
   * Construct a new immutable {@code OAuth2CredentialFlowState} instance.
   * @param connectorId The value for the {@code connectorId} attribute
   * @param key The value for the {@code key} attribute
   * @param providerId The value for the {@code providerId} attribute
   * @param redirectUrl The value for the {@code redirectUrl} attribute
   * @param returnUrl The value for the {@code returnUrl} attribute
   * @param accessGrant The value for the {@code accessGrant} attribute
   * @param code The value for the {@code code} attribute
   * @param state The value for the {@code state} attribute
   * @return An immutable OAuth2CredentialFlowState instance
   */
  public static OAuth2CredentialFlowState of(String connectorId, String key, String providerId, String redirectUrl, URI returnUrl, AccessGrant accessGrant, String code, String state) {
    return validate(new ImmutableOAuth2CredentialFlowState(connectorId, key, providerId, redirectUrl, returnUrl, accessGrant, code, state));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableOAuth2CredentialFlowState validate(ImmutableOAuth2CredentialFlowState instance) {
    Set<ConstraintViolation<ImmutableOAuth2CredentialFlowState>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link OAuth2CredentialFlowState} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable OAuth2CredentialFlowState instance
   */
  public static OAuth2CredentialFlowState copyOf(OAuth2CredentialFlowState instance) {
    if (instance instanceof ImmutableOAuth2CredentialFlowState) {
      return (ImmutableOAuth2CredentialFlowState) instance;
    }
    return new OAuth2CredentialFlowState.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link OAuth2CredentialFlowState OAuth2CredentialFlowState}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "OAuth2CredentialFlowState", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private @Nullable String connectorId;
    private @Nullable String key;
    private @Nullable String providerId;
    private @Nullable String redirectUrl;
    private @Nullable URI returnUrl;
    private @Nullable AccessGrant accessGrant;
    private @Nullable String code;
    private @Nullable String state;

    /**
     * Creates a builder for {@link OAuth2CredentialFlowState OAuth2CredentialFlowState} instances.
     * <pre>
     * new OAuth2CredentialFlowState.Builder()
     *    .connectorId(String) // optional {@link OAuth2CredentialFlowState#getConnectorId() connectorId}
     *    .key(String | null) // nullable {@link OAuth2CredentialFlowState#getKey() key}
     *    .providerId(String | null) // nullable {@link OAuth2CredentialFlowState#getProviderId() providerId}
     *    .redirectUrl(String | null) // nullable {@link OAuth2CredentialFlowState#getRedirectUrl() redirectUrl}
     *    .returnUrl(java.net.URI | null) // nullable {@link OAuth2CredentialFlowState#getReturnUrl() returnUrl}
     *    .accessGrant(org.springframework.social.oauth2.AccessGrant | null) // nullable {@link OAuth2CredentialFlowState#getAccessGrant() accessGrant}
     *    .code(String | null) // nullable {@link OAuth2CredentialFlowState#getCode() code}
     *    .state(String | null) // nullable {@link OAuth2CredentialFlowState#getState() state}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof OAuth2CredentialFlowState.Builder)) {
        throw new UnsupportedOperationException("Use: new OAuth2CredentialFlowState.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.server.credential.CredentialFlowState} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final OAuth2CredentialFlowState.Builder createFrom(CredentialFlowState instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.server.credential.OAuth2CredentialFlowState} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final OAuth2CredentialFlowState.Builder createFrom(OAuth2CredentialFlowState instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuth2CredentialFlowState.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof CredentialFlowState) {
        CredentialFlowState instance = (CredentialFlowState) object;
        String redirectUrlValue = instance.getRedirectUrl();
        if (redirectUrlValue != null) {
          redirectUrl(redirectUrlValue);
        }
        URI returnUrlValue = instance.getReturnUrl();
        if (returnUrlValue != null) {
          returnUrl(returnUrlValue);
        }
        connectorId(instance.getConnectorId());
        String providerIdValue = instance.getProviderId();
        if (providerIdValue != null) {
          providerId(providerIdValue);
        }
        String keyValue = instance.getKey();
        if (keyValue != null) {
          key(keyValue);
        }
      }
      if (object instanceof OAuth2CredentialFlowState) {
        OAuth2CredentialFlowState instance = (OAuth2CredentialFlowState) object;
        String codeValue = instance.getCode();
        if (codeValue != null) {
          code(codeValue);
        }
        AccessGrant accessGrantValue = instance.getAccessGrant();
        if (accessGrantValue != null) {
          accessGrant(accessGrantValue);
        }
        String stateValue = instance.getState();
        if (stateValue != null) {
          state(stateValue);
        }
      }
    }

    /**
     * Initializes the value for the {@link OAuth2CredentialFlowState#getConnectorId() connectorId} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link OAuth2CredentialFlowState#getConnectorId() connectorId}.</em>
     * @param connectorId The value for connectorId 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connectorId")
    public final OAuth2CredentialFlowState.Builder connectorId(String connectorId) {
      this.connectorId = Objects.requireNonNull(connectorId, "connectorId");
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth2CredentialFlowState#getKey() key} attribute.
     * @param key The value for key (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("key")
    public final OAuth2CredentialFlowState.Builder key(String key) {
      this.key = key;
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth2CredentialFlowState#getProviderId() providerId} attribute.
     * @param providerId The value for providerId (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("providerId")
    public final OAuth2CredentialFlowState.Builder providerId(String providerId) {
      this.providerId = providerId;
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth2CredentialFlowState#getRedirectUrl() redirectUrl} attribute.
     * @param redirectUrl The value for redirectUrl (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("redirectUrl")
    @JsonIgnore
    public final OAuth2CredentialFlowState.Builder redirectUrl(String redirectUrl) {
      this.redirectUrl = redirectUrl;
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth2CredentialFlowState#getReturnUrl() returnUrl} attribute.
     * @param returnUrl The value for returnUrl (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("returnUrl")
    public final OAuth2CredentialFlowState.Builder returnUrl(URI returnUrl) {
      this.returnUrl = returnUrl;
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth2CredentialFlowState#getAccessGrant() accessGrant} attribute.
     * @param accessGrant The value for accessGrant (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("accessGrant")
    @JsonDeserialize(converter = OAuth2CredentialFlowState.AccessGrantConverter.class)
    public final OAuth2CredentialFlowState.Builder accessGrant(AccessGrant accessGrant) {
      this.accessGrant = accessGrant;
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth2CredentialFlowState#getCode() code} attribute.
     * @param code The value for code (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("code")
    public final OAuth2CredentialFlowState.Builder code(String code) {
      this.code = code;
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuth2CredentialFlowState#getState() state} attribute.
     * @param state The value for state (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("state")
    public final OAuth2CredentialFlowState.Builder state(String state) {
      this.state = state;
      return (OAuth2CredentialFlowState.Builder) this;
    }

    /**
     * Builds a new {@link OAuth2CredentialFlowState OAuth2CredentialFlowState}.
     * @return An immutable instance of OAuth2CredentialFlowState
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public OAuth2CredentialFlowState build() {
      return ImmutableOAuth2CredentialFlowState.validate(new ImmutableOAuth2CredentialFlowState(this));
    }
  }
}
