/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.credential;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link AcquisitionResponse.State}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new AcquisitionResponse.State.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableState.of()}.
 */
@Generated(from = "AcquisitionResponse.State", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
final class ImmutableState implements AcquisitionResponse.State {
  private final AcquisitionResponse.State.Persist persist;
  private final String spec;

  private ImmutableState(AcquisitionResponse.State.Persist persist, String spec) {
    this.persist = persist;
    this.spec = spec;
  }

  /**
   * @return The value of the {@code persist} attribute
   */
  @JsonProperty("persist")
  @Override
  public AcquisitionResponse.State.Persist persist() {
    return persist;
  }

  /**
   * @return The value of the {@code spec} attribute
   */
  @JsonProperty("spec")
  @Override
  public String spec() {
    return spec;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionResponse.State#persist() persist} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for persist (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableState withPersist(AcquisitionResponse.State.Persist value) {
    if (this.persist == value) return this;
    if (Objects.equals(this.persist, value)) return this;
    return validate(new ImmutableState(value, this.spec));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link AcquisitionResponse.State#spec() spec} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for spec (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableState withSpec(String value) {
    if (Objects.equals(this.spec, value)) return this;
    return validate(new ImmutableState(this.persist, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableState} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableState
        && equalTo((ImmutableState) another);
  }

  private boolean equalTo(ImmutableState another) {
    return Objects.equals(persist, another.persist)
        && Objects.equals(spec, another.spec);
  }

  /**
   * Computes a hash code from attributes: {@code persist}, {@code spec}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(persist);
    h += (h << 5) + Objects.hashCode(spec);
    return h;
  }

  /**
   * Prints the immutable value {@code State} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "State{"
        + "persist=" + persist
        + ", spec=" + spec
        + "}";
  }

  /**
   * Construct a new immutable {@code State} instance.
   * @param persist The value for the {@code persist} attribute
   * @param spec The value for the {@code spec} attribute
   * @return An immutable State instance
   */
  public static AcquisitionResponse.State of(AcquisitionResponse.State.Persist persist, String spec) {
    return validate(new ImmutableState(persist, spec));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableState validate(ImmutableState instance) {
    Set<ConstraintViolation<ImmutableState>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link AcquisitionResponse.State} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable State instance
   */
  public static AcquisitionResponse.State copyOf(AcquisitionResponse.State instance) {
    if (instance instanceof ImmutableState) {
      return (ImmutableState) instance;
    }
    return new AcquisitionResponse.State.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link AcquisitionResponse.State State}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "AcquisitionResponse.State", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private @Nullable AcquisitionResponse.State.Persist persist;
    private @Nullable String spec;

    /**
     * Creates a builder for {@link AcquisitionResponse.State State} instances.
     * <pre>
     * new AcquisitionResponse.State.Builder()
     *    .persist(io.syndesis.server.credential.AcquisitionResponse.State.Persist | null) // nullable {@link AcquisitionResponse.State#persist() persist}
     *    .spec(String | null) // nullable {@link AcquisitionResponse.State#spec() spec}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof AcquisitionResponse.State.Builder)) {
        throw new UnsupportedOperationException("Use: new AcquisitionResponse.State.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code State} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final AcquisitionResponse.State.Builder createFrom(AcquisitionResponse.State instance) {
      Objects.requireNonNull(instance, "instance");
      AcquisitionResponse.State.Persist persistValue = instance.persist();
      if (persistValue != null) {
        persist(persistValue);
      }
      String specValue = instance.spec();
      if (specValue != null) {
        spec(specValue);
      }
      return (AcquisitionResponse.State.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionResponse.State#persist() persist} attribute.
     * @param persist The value for persist (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("persist")
    public final AcquisitionResponse.State.Builder persist(AcquisitionResponse.State.Persist persist) {
      this.persist = persist;
      return (AcquisitionResponse.State.Builder) this;
    }

    /**
     * Initializes the value for the {@link AcquisitionResponse.State#spec() spec} attribute.
     * @param spec The value for spec (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("spec")
    public final AcquisitionResponse.State.Builder spec(String spec) {
      this.spec = spec;
      return (AcquisitionResponse.State.Builder) this;
    }

    /**
     * Builds a new {@link AcquisitionResponse.State State}.
     * @return An immutable instance of State
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public AcquisitionResponse.State build() {
      return ImmutableState.validate(new ImmutableState(persist, spec));
    }
  }
}
